import imp
import itertools
import os
import sys
import platform
import logging
import importlib

_logger = logging.getLogger(__name__)
_HAS_NEW_IMPORTLIB = (sys.version_info > (3, 3))

if _HAS_NEW_IMPORTLIB:
    from importlib.machinery import SourceFileLoader

class NoInitFileFound(Exception):
    pass

def import_file(filename):
    module_name = _setup_module_name_for_import(filename)
    if _HAS_NEW_IMPORTLIB:
        if os.path.isdir(filename):
            filename = os.path.join(filename, "__init__.py")
        return SourceFileLoader(module_name, filename).load_module()
    return __import__(module_name, fromlist=[''])

_package_name_generator = ('_{0}'.format(x) for x in itertools.count())

def _generate_package_name():
    for suggested in _package_name_generator:
        if not _package_name_exists(suggested):
            return suggested

def _package_name_exists(pkg_name):
    return pkg_name in sys.modules

def _setup_module_name_for_import(filename):
    return _create_new_module_name(filename)

_cached_package_names = {}

def _create_new_module_name(filename):
    _logger.debug("Creating new package for %s", filename)
    nonpackage_dir, remainder = _split_nonpackage_dir(filename)
    _logger.debug("After split: %s, %s", nonpackage_dir, remainder)
    package_name = _cached_package_names.get(nonpackage_dir, None)
    if package_name is None:
        package_name = _generate_package_name()
        sys.modules[package_name] = _create_package_module(package_name, nonpackage_dir)
        _cached_package_names[nonpackage_dir] = package_name
    return '{0}.{1}'.format(package_name, remainder)

def _split_nonpackage_dir(path):
    if not os.path.isdir(path):
        nonpackage_dir, module = os.path.split(os.path.normpath(os.path.abspath(path)))
        module = _make_module_name(module).split(".")
    else:
        nonpackage_dir = path
        module = []
    while os.path.isfile(os.path.join(nonpackage_dir, "__init__.py")):
        if '.' in os.path.split(nonpackage_dir)[-1]:
            # we cannot import from such packages, stop traversing upwards...
            break
        nonpackage_dir, current_component = os.path.split(nonpackage_dir)
        module.insert(0, current_component)
        _logger.debug("Now at %s, %s", nonpackage_dir, module)
    if not module:
        raise NoInitFileFound("Could not find __init__.py file in {0}".format(path))
    return nonpackage_dir, ".".join(module)

def _make_module_name(filename):
    assert filename.endswith('.py') or filename.endswith('.pyc')
    return filename.rsplit(".", 1)[0].replace(os.path.sep, ".")

def _create_package_module(name, path):
    imp.acquire_lock()
    try:
        if _HAS_NEW_IMPORTLIB:
            # the package import machinery works a bit differently in
            # python 3.3
            returned = imp.new_module(name)
            returned.__path__ = [path]
            sys.modules[name] = returned
        else:
            returned = imp.load_module(name, None, path, ('', '', imp.PKG_DIRECTORY))
    finally:
        imp.release_lock()
    return returned



import itertools
import uuid
import tempfile
import os
import sys
try:
    from unittest2 import TestCase
except ImportError:
    from unittest import TestCase
import emport

_filename_generator = ("module{0}.py".format(i) for i in itertools.count())

class ImportingTest(TestCase):
    def setUp(self):
        super(ImportingTest, self).setUp()
        self._old_sys_modules = sys.modules.copy()
        self._old_sys_path = sys.path[:]
        self.root = tempfile.mkdtemp()
    def tearDown(self):
        sys.modules = self._old_sys_modules
        sys.path[:] = self._old_sys_path
        super(ImportingTest, self).tearDown()
    def test__import_from_non_package(self):
        self._import_and_verify(self._create_modules([tempfile.mkdtemp()]))
    def test__multiple_imports_from_same_non_package(self):
        filenames_and_expected_values = self._create_modules([self.root, self.root])
        module1, module2 = self._import_and_verify(filenames_and_expected_values)
        self.assertEquals(_PACKAGE_NAME(module1), _PACKAGE_NAME(module2))
        self.assertIsNot(module1, module2)
    def test__importing_sub_package(self):
        subpackage_dir = os.path.join(self.root, "subpackage")
        os.makedirs(subpackage_dir)
        _touch(subpackage_dir, "__init__.py")
        full_filename, expected_value = self._create_module(subpackage_dir, "module.py")
        module = emport.import_file(full_filename)
        self.assertTrue(module.__name__.endswith(".subpackage.module"))
    def test__importing_sub_package_and_subdir(self):
        to_verify = self._create_modules([self.root])
        [module] = self._import_and_verify(to_verify)
        package_name = _PACKAGE_NAME(module)
        to_verify = self._create_modules([os.path.join(self.root, "sub")])
        _touch(self.root, "sub", "__init__.py")
        [sub_module] = self._import_and_verify(to_verify)
        self.assertEquals(_PACKAGE_NAME(module) + ".sub",
                          _PACKAGE_NAME(sub_module))
    def test__importing_different_directories_same_escaping(self):
        dir1 = os.path.join(self.root, "pkg+")
        dir2 = os.path.join(self.root, "pkg-")
        filenames_and_expected_values = self._create_modules([dir1, dir2])
        module1, module2 = self._import_and_verify(filenames_and_expected_values)
        self.assertNotEquals(_PACKAGE_NAME(module1),
                             _PACKAGE_NAME(module2))
    def test__importing_dotted_name(self):
        path_components = ["a", "b.c", "d"]
        path = self.root
        for c in path_components:
            path = os.path.join(path, c)
            os.makedirs(path)
            _touch(path, "__init__.py")
        filename, expected_value = self._create_module(path, "module.py")
        self._import_and_verify([(filename, expected_value)])
    def test__importing_directory_no_init_file(self):
        with self.assertRaises(emport.NoInitFileFound):
            emport.import_file(self.root)
    def test__importing_directory_directly(self):
        self._test__importing_directory(False)
    def test__importing_directory_through_init_py(self):
        self._test__importing_directory(True)
    def _test__importing_directory(self, init_py_directly):
        directory = os.path.join(self.root, "pkg")
        os.makedirs(directory)
        _, expected_value = self._create_module(directory, "__init__.py")
        if init_py_directly:
            module = emport.import_file(os.path.join(directory, "__init__.py"))
        else:
            module = emport.import_file(directory)
        self.assertRaises(module.__name__.endswith(".pkg"))
        self.assertEquals(module.value, expected_value)
    def _create_modules(self, directories):
        returned = []
        for directory in directories:
            if not os.path.isdir(directory):
                os.makedirs(directory)
            returned.append(self._create_module(directory))
        return returned
    def _create_module(self, directory, filename=None):
        id = repr(uuid.uuid1())
        if filename is None:
            filename = next(_filename_generator)
        full_filename = os.path.join(directory, filename)
        with open(full_filename, "w") as f:
            f.write("value = {0!r}".format(id))
        return full_filename, id
    def _import_and_verify(self, filenames_and_expected_values):
        returned = []
        for filename, expected_value in filenames_and_expected_values:
            module = emport.import_file(filename)
            self.assertEquals(module.value, expected_value)
            returned.append(module)
        return returned

def _touch(*p):
    with open(os.path.join(*p), "a"):
        pass

def _PACKAGE_NAME(m):
    return m.__name__.rsplit(".", 1)[0]



import os
import sys
from setuptools import setup, find_packages

with open(os.path.join(os.path.dirname(__file__), "emport", "__version__.py")) as version_file:
    exec(version_file.read()) # pylint: disable=W0122

_INSTALL_REQUIERS = []
if sys.version_info < (2, 7):
    _INSTALL_REQUIERS.append("importlib")

setup(name="emport",
      classifiers = [
          "Programming Language :: Python :: 2.6",
          "Programming Language :: Python :: 2.7",
          "Programming Language :: Python :: 3.3",
          ],
      description="Utility library for performing programmatic imports",
      license="BSD",
      author="Rotem Yaari",
      author_email="vmalloc@gmail.com",
      version=__version__, # pylint: disable=E0602
      packages=find_packages(exclude=["tests"]),
      url="https://github.com/vmalloc/emport",
      install_requires=_INSTALL_REQUIERS,
      scripts=[],
      namespace_packages=[]
      )
